import unittest
from psycopg2cffi.tests import psycopg2_tests

def suite():
    suite = unittest.TestSuite()
    suite.addTest(psycopg2_tests.test_suite())
    return suite


if __name__ == '__main__':
    unittest.main(defaultTest='suite')



"""psycopg2cffi -- global constants

This module can be imported from everywhere without problems of cross imports.
"""

# Isolation level values.
ISOLATION_LEVEL_AUTOCOMMIT = 0
ISOLATION_LEVEL_READ_UNCOMMITTED = 4
ISOLATION_LEVEL_READ_COMMITTED = 1
ISOLATION_LEVEL_REPEATABLE_READ = 2
ISOLATION_LEVEL_SERIALIZABLE = 3

# psycopg connection status values.
STATUS_SETUP = 0
STATUS_READY = 1
STATUS_BEGIN = 2
STATUS_SYNC = 3     # currently unused
STATUS_ASYNC = 4    # currently unused
STATUS_PREPARED = 5
STATUS_CONNECTING = 20
STATUS_DATESTYLE = 21

# This is a usefull mnemonic to check if the connection is in a transaction
STATUS_IN_TRANSACTION = STATUS_BEGIN

# psycopg asynchronous connection polling values
POLL_OK = 0
POLL_READ = 1
POLL_WRITE = 2
POLL_ERROR = 3

# Backend transaction status values.
TRANSACTION_STATUS_IDLE = 0
TRANSACTION_STATUS_ACTIVE = 1
TRANSACTION_STATUS_INTRANS = 2
TRANSACTION_STATUS_INERROR = 3
TRANSACTION_STATUS_UNKNOWN = 4


ASYNC_DONE = 0
ASYNC_READ = 1
ASYNC_WRITE = 2

LOBJECT_READ = 1
LOBJECT_WRITE = 2
LOBJECT_TEXT = 4
LOBJECT_BINARY = 8



"""psycopg extensions to the DBAPI-2.0

This module holds all the extensions to the DBAPI-2.0 provided by psycopg.

- `connection` -- the new-type inheritable connection class
- `cursor` -- the new-type inheritable cursor class
- `lobject` -- the new-type inheritable large object class
- `adapt()` -- exposes the PEP-246_ compatible adapting mechanism used
  by psycopg to adapt Python types to PostgreSQL ones

.. _PEP-246: http://www.python.org/peps/pep-0246.html
"""

from __future__ import unicode_literals

# psycopg/extensions.py - DBAPI-2.0 extensions specific to psycopg
#
# Copyright (C) 2003-2010 Federico Di Gregorio  <fog@debian.org>
#
# psycopg2 is free software: you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In addition, as a special exception, the copyright holders give
# permission to link this program with the OpenSSL library (or with
# modified versions of OpenSSL that use the same license as OpenSSL),
# and distribute linked combinations including the two.
#
# You must obey the GNU Lesser General Public License in all respects for
# all of the code used other than OpenSSL.
#
# psycopg2 is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
# License for more details.
import sys as _sys

from psycopg2cffi._impl import connection as _connection
from psycopg2cffi._impl.adapters import adapt, adapters
from psycopg2cffi._impl.adapters import Binary, Boolean, Int, Float
from psycopg2cffi._impl.adapters import QuotedString, AsIs, ISQLQuote
from psycopg2cffi._impl.connection import Connection as connection
from psycopg2cffi._impl.consts import *
from psycopg2cffi._impl.cursor import Cursor as cursor
from psycopg2cffi._impl.encodings import encodings
from psycopg2cffi._impl.exceptions import QueryCanceledError
from psycopg2cffi._impl.exceptions import TransactionRollbackError
from psycopg2cffi._impl.exceptions import Diagnostics
from psycopg2cffi._impl.lobject import LargeObject as lobject
from psycopg2cffi._impl.notify import Notify
from psycopg2cffi._impl.typecasts import (
    UNICODE, INTEGER, LONGINTEGER, BOOLEAN, FLOAT, TIME, DATE, INTERVAL,
    DECIMAL,
    BINARYARRAY, BOOLEANARRAY, DATEARRAY, DATETIMEARRAY, DECIMALARRAY,
    FLOATARRAY, INTEGERARRAY, INTERVALARRAY, LONGINTEGERARRAY, ROWIDARRAY,
    STRINGARRAY, TIMEARRAY, UNICODEARRAY)
from psycopg2cffi._impl.typecasts import string_types, binary_types
from psycopg2cffi._impl.typecasts import new_type, new_array_type, register_type
from psycopg2cffi._impl.xid import Xid

# These are looked for by the test suite to make the tests run
from psycopg2cffi._impl.typecasts import DATETIME as PYDATETIME
PYDATE = DATE
PYTIME = TIME
PYINTERVAL = INTERVAL

# Return bytes from a string
if _sys.version_info[0] < 3:
    def b(s):
        return s
else:
    def b(s):
        return s.encode('utf8')

def register_adapter(typ, callable):
    """Register 'callable' as an ISQLQuote adapter for type 'typ'."""
    adapters[(typ, ISQLQuote)] = callable


# The SQL_IN class is the official adapter for tuples starting from 2.0.6.
class SQL_IN(object):
    """Adapt any iterable to an SQL quotable object."""
    def __init__(self, seq):
        self._seq = seq
        self._conn = None

    def prepare(self, conn):
        self._conn = conn

    def getquoted(self):
        # this is the important line: note how every object in the
        # list is adapted and then how getquoted() is called on it
        pobjs = [adapt(o) for o in self._seq]
        if self._conn is not None:
            for obj in pobjs:
                if hasattr(obj, 'prepare'):
                    obj.prepare(self._conn)
        qobjs = [o.getquoted() for o in pobjs]
        return b'(' + b', '.join(qobjs) + b')'

    def __str__(self):
        return str(self.getquoted())


class NoneAdapter(object):
    """Adapt None to NULL.

    This adapter is not used normally as a fast path in mogrify uses NULL,
    but it makes easier to adapt composite types.
    """
    def __init__(self, obj):
        pass

    def getquoted(self, _null=b"NULL"):
        return _null


# Create default json typecasters for PostgreSQL 9.2 oids
from psycopg2cffi._json import register_default_json, register_default_jsonb

try:
    JSON, JSONARRAY = register_default_json()
    JSONB, JSONBARRAY = register_default_jsonb()
except ImportError:
    pass

del register_default_json, register_default_jsonb


# Create default Range typecasters
from psycopg2cffi._range import Range
del Range

def set_wait_callback(f):
    _connection._green_callback = f


def get_wait_callback():
    return _connection._green_callback



from __future__ import unicode_literals

import re
import decimal
import datetime
from time import localtime
import six

from psycopg2cffi._impl.libpq import libpq, ffi
from psycopg2cffi._impl.adapters import bytes_to_ascii, ascii_to_bytes
from psycopg2cffi._impl.exceptions import DataError

# Typecasters accept bytes and return python objects.
# This only applies to our internal typecasers - user-defined ones
# accept unicode, to mimic psycopg2 behaviour

string_types = {}

binary_types = {}


class Type(object):
    def __init__(self, name, values, caster=None, py_caster=None, accept_unicode=True):
        self.name = name
        self.values = values
        self.caster = caster
        self.py_caster = py_caster
        self.accept_unicode = accept_unicode

    def __eq__(self, other):
        return other in self.values

    def cast(self, value, cursor=None, length=None):
        if self.py_caster is not None:
            # py_caster-s are part of external api and so accept unicode
            if isinstance(value, six.binary_type) and self.accept_unicode:
                value = value.decode(cursor._conn._py_enc)
            return self.py_caster(value, cursor)
        return self.caster(value, length, cursor)

    __call__ = cast


def register_type(type_obj, scope=None):
    typecasts = string_types
    if scope:
        from psycopg2cffi._impl.connection import Connection
        from psycopg2cffi._impl.cursor import Cursor

        if isinstance(scope, Connection):
            typecasts = scope._typecasts
        elif isinstance(scope, Cursor):
            typecasts = scope._typecasts
        else:
            typecasts = None

    for value in type_obj.values:
        typecasts[value] = type_obj


def new_type(values, name, castobj, accept_unicode=True):
    return Type(name, values, py_caster=castobj, accept_unicode=accept_unicode)


def new_array_type(values, name, baseobj):
    caster = parse_array(baseobj)
    return Type(name, values, caster=caster)


def typecast(caster, value, length, cursor):
    return caster.cast(value, cursor, length)


def parse_unknown(value, length, cursor):
    if value is None:
        return None
    if value != b'{}':
        # FIXME hmm not sure
        if six.PY3 and isinstance(value, six.binary_type):
            return parse_unicode(value, length, cursor)
        return value
    else:
        return []


if six.PY3:
    def parse_string(value, length, cursor):
        return value.decode(cursor.connection._py_enc) \
                if value is not None else None
else:
    def parse_string(value, length, cursor):
        return value


if six.PY3:
    def parse_longinteger(value, length, cursor):
        return int(value) if value is not None else None
else:
    def parse_longinteger(value, length, cursor):
        return long(value) if value is not None else None


def parse_integer(value, length, cursor):
    return int(value) if value is not None else None


def parse_float(value, length, cursor):
    return float(value) if value is not None else None


def parse_decimal(value, length, cursor):
    if value is None:
        return None
    if isinstance(value, six.binary_type):
        value = bytes_to_ascii(value)
    return decimal.Decimal(value)


def parse_binary(value, length, cursor):
    if value is None:
        return None

    to_length = ffi.new('size_t *')
    s = libpq.PQunescapeBytea(
            ffi.new('unsigned char[]', value), to_length)
    try:
        res = ffi.buffer(s, to_length[0])[:]
    finally:
        libpq.PQfreemem(s)
    return memoryview(res) if six.PY3 else buffer(res)


def parse_boolean(value, length, cursor):
    """Typecast the postgres boolean to a python boolean.

    Postgres returns the boolean as a string with 'true' or 'false'

    """
    return value[:1] == b"t" if value is not None else None


class parse_array(object):
    """Parse an array of a items using an configurable caster for the items

    The array syntax is defined as::

        '{ val1 delim val2 delim ... }'

    A two-dimensional array with string items is defined as::

        '{{"meeting", "lunch"}, {"training", "presentation"}}'

    """
    def __init__(self, caster):
        self._caster = caster

    def cast(self, value, length, cursor):
        if value is None:
            return None

        s = value
        if not (len(s) >= 2 and  s[:1] == b"{" and s[-1:] == b"}"):
            raise DataError("malformed array")

        i = 1
        array = []
        stack = [array]
        value_length = len(s) - 1
        while i < value_length:
            si = s[i:i+1]
            if si == b'{':
                sub_array = []
                array.append(sub_array)
                stack.append(sub_array)
                if len(stack) > 16:
                    raise DataError("excessive array dimensions")

                array = sub_array
                i += 1
            elif si == b'}':
                stack.pop()
                if not stack:
                    raise DataError("unbalanced braces in array")
                array = stack[-1]
                i += 1
            elif si in b', ':
                i += 1
            else:
                # Number of quotes, this will always be 0 or 2 (int vs str)
                quotes = 0

                # Whether or not the next char should be escaped
                escape_char = False

                buf = []
                while i < value_length:
                    si = s[i:i+1]
                    if not escape_char:
                        if si == b'"':
                            quotes += 1
                        elif si == b'\\':
                            escape_char = True
                        elif quotes % 2 == 0 and (si == b'}' or si == b','):
                            break
                        else:
                            buf.append(si)
                    else:
                        escape_char = False
                        buf.append(si)

                    i += 1

                str_buf = b''.join(buf)
                if len(str_buf) == 4 and str_buf.lower() == b'null':
                    val = typecast(self._caster, None, 0, cursor)
                else:
                    val = typecast(self._caster, str_buf, len(str_buf), cursor)
                array.append(val)
        return stack[-1]

    __call__ = cast


def parse_unicode(value, length, cursor):
    """Decode the given value with the connection encoding"""
    if value is None:
        return None
    elif isinstance(value, six.text_type):
        # This can occur when we use our internal typecaster
        # in register_type, so it accepts unicode, not bytes.
        return value
    else:
        return value.decode(cursor._conn._py_enc)


def _parse_time_to_args(value, cursor):
    """Return arguemnts for datetime.time constructor

    The given value is in the format of `16:28:09.506488+01`

    """
    hour, minute, second = value.split(b':', 2)

    sign = 0
    tzinfo = None
    timezone = None
    if b'-' in second:
        sign = -1
        second, timezone = second.split(b'-')
    elif b'+' in second:
        sign = 1
        second, timezone = second.split(b'+')

    if not cursor.tzinfo_factory is None and sign:
        parts = timezone.split(b':')
        tz_min = 60 * int(parts[0])
        if len(parts) > 1:
            tz_min += int(parts[1])
        if len(parts) > 2 and int(parts[2]) >= 30:
            tz_min += 1
        tzinfo = cursor.tzinfo_factory(sign * tz_min)

    if b'.' in second:
        second, frac = second.split(b'.')
        micros = int((frac + (b'0' * (6 - len(frac))))[:6])
    else:
        micros = 0

    return int(hour), int(minute), int(second), micros, tzinfo


def parse_datetime(value, length, cursor):
    if value is None:
        return None
    if isinstance(value, six.text_type):
        value = ascii_to_bytes(value)
    if value == b'infinity':
        return datetime.datetime.max
    elif value == b'-infinity':
        return datetime.datetime.min

    try:
        date, time = value.split(b' ')
        date_args = date.split(b'-')
        return datetime.datetime(
                int(date_args[0]),
                int(date_args[1]),
                int(date_args[2]),
                *_parse_time_to_args(time, cursor))
    except (TypeError, ValueError):
        if value.endswith(b'BC'):
            raise ValueError('BC dates not supported')
        raise DataError("bad datetime: '%s'" % bytes_to_ascii(value))


def parse_date(value, length, cursor):
    if value is None:
        return None
    if isinstance(value, six.text_type):
        value = ascii_to_bytes(value)
    if value == b'infinity':
        return datetime.date.max
    elif value == b'-infinity':
        return datetime.date.min

    try:
        return datetime.date(*[int(x) for x in value.split(b'-')])
    except (TypeError, ValueError):
        if value.endswith(b'BC'):
            raise ValueError('BC dates not supported')
        raise DataError("bad datetime: '%s'" % bytes_to_ascii(value))


def parse_time(value, length, cursor):
    if value is None:
        return None
    if isinstance(value, six.text_type):
        value = ascii_to_bytes(value)
    try:
        return datetime.time(*_parse_time_to_args(value, cursor))
    except (TypeError, ValueError):
        raise DataError("bad datetime: '%s'" % value)


_re_interval = re.compile(br"""
    (?:(-?\+?\d+)\sy\w+\s?)?    # year
    (?:(-?\+?\d+)\sm\w+\s?)?    # month
    (?:(-?\+?\d+)\sd\w+\s?)?    # day
    (?:(-?\+?)(\d+):(\d+):(\d+) # +/- hours:mins:secs
        (?:\.(\d+))?)?          # second fraction
    """, re.VERBOSE)

def parse_interval(value, length, cursor):
    """Typecast an interval to a datetime.timedelta instance.

    For example, the value '2 years 1 mon 3 days 10:01:39.100' is converted
    to `datetime.timedelta(763, 36099, 100)`.

    """
    if value is None:
        return None
    if isinstance(value, six.text_type):
        value = ascii_to_bytes(value)

    m = _re_interval.match(value)
    if not m:
        raise ValueError("failed to parse interval: '%s'" % value)

    years, months, days, sign, hours, mins, secs, frac = m.groups()

    days = int(days) if days is not None else 0
    if months is not None:
        days += int(months) * 30
    if years is not None:
        days += int(years) * 365

    if hours is not None:
        secs = int(hours) * 3600 + int(mins) * 60 + int(secs)
        if frac is not None:
            micros = int((frac + (b'0' * (6 - len(frac))))[:6])
        else:
            micros = 0

        if sign == b'-':
            secs = -secs
            micros = -micros
    else:
        secs = micros = 0

    return datetime.timedelta(days, secs, micros)


def Date(year, month, day):
    from psycopg2cffi.extensions.adapters import DateTime
    date = datetime.date(year, month, day)
    return DateTime(date)


def DateFromTicks(ticks):
    tm = localtime()
    return Date(tm.tm_year, tm.tm_mon, tm.tm_mday)


def Binary(obj):
    from psycopg2cffi.extensions.adapters import Binary
    return Binary(obj)


def _default_type(name, oids, caster):
    """Shortcut to register internal types"""
    type_obj = Type(name, oids, caster)
    register_type(type_obj)
    return type_obj


# DB API 2.0 types
BINARY = _default_type('BINARY', [17], parse_binary)
DATETIME = _default_type('DATETIME',  [1114, 1184, 704, 1186], parse_datetime)
NUMBER = _default_type('NUMBER', [20, 33, 21, 701, 700, 1700], parse_float)
ROWID = _default_type('ROWID', [26], parse_integer)
STRING = _default_type('STRING', [19, 18, 25, 1042, 1043], parse_string)

# Register the basic typecasters
BOOLEAN = _default_type('BOOLEAN', [16], parse_boolean)
DATE = _default_type('DATE', [1082], parse_date)
DECIMAL = _default_type('DECIMAL', [1700], parse_decimal)
FLOAT = _default_type('FLOAT', [701, 700], parse_float)
INTEGER = _default_type('INTEGER', [23, 21], parse_integer)
INTERVAL = _default_type('INTERVAL', [704, 1186], parse_interval)
LONGINTEGER = _default_type('LONGINTEGER', [20], parse_longinteger)
TIME = _default_type('TIME', [1083, 1266], parse_time)
UNKNOWN = _default_type('UNKNOWN', [705], parse_unknown)

# Array types
BINARYARRAY = _default_type(
    'BINARYARRAY', [1001], parse_array(BINARY))
BOOLEANARRAY = _default_type(
    'BOOLEANARRAY', [1000], parse_array(BOOLEAN))
DATEARRAY = _default_type(
    'DATEARRAY', [1182], parse_array(DATE))
DATETIMEARRAY = _default_type(
    'DATETIMEARRAY', [1115, 1185], parse_array(DATETIME))
DECIMALARRAY = _default_type(
    'DECIMALARRAY', [1231], parse_array(DECIMAL))
FLOATARRAY = _default_type(
    'FLOATARRAY', [1017, 1021, 1022], parse_array(FLOAT))
INTEGERARRAY = _default_type(
    'INTEGERARRAY', [1005, 1006, 1007], parse_array(INTEGER))
INTERVALARRAY = _default_type(
    'INTERVALARRAY', [1187], parse_array(INTERVAL))
LONGINTEGERARRAY = _default_type(
    'LONGINTEGERARRAY', [1016], parse_array(LONGINTEGER))
ROWIDARRAY = _default_type(
    'ROWIDARRAY', [1013, 1028], parse_array(ROWID))
STRINGARRAY = _default_type(
    'STRINGARRAY', [1002, 1003, 1009, 1014, 1015], parse_array(STRING))
TIMEARRAY = _default_type(
    'TIMEARRAY', [1183, 1270], parse_array(TIME))


UNICODE = Type('UNICODE', [19, 18, 25, 1042, 1043], parse_unicode)
UNICODEARRAY = Type('UNICODEARRAY', [1002, 1003, 1009, 1014, 1015],
    parse_array(UNICODE))



import datetime
from time import localtime

from psycopg2cffi import extensions
from psycopg2cffi import tz
from psycopg2cffi._impl.adapters import Binary, Date, Time, Timestamp
from psycopg2cffi._impl.adapters import DateFromTicks, TimeFromTicks
from psycopg2cffi._impl.adapters import TimestampFromTicks
from psycopg2cffi._impl.connection import _connect
from psycopg2cffi._impl.exceptions import *
from psycopg2cffi._impl.typecasts import BINARY, DATETIME, NUMBER, ROWID, STRING

__version__ = '2.8.1'
apilevel = '2.0'
paramstyle = 'pyformat'
threadsafety = 2

import psycopg2cffi.extensions as _ext
_ext.register_adapter(tuple, _ext.SQL_IN)
_ext.register_adapter(type(None), _ext.NoneAdapter)


import re

def _param_escape(s,
        re_escape=re.compile(r"([\\'])"),
        re_space=re.compile(r'\s')):
    """
    Apply the escaping rule required by PQconnectdb
    """
    if not s: return "''"

    s = re_escape.sub(r'\\\1', s)
    if re_space.search(s):
        s = "'" + s + "'"

    return s

del re


def connect(dsn=None,
        database=None, user=None, password=None, host=None, port=None,
        connection_factory=None, cursor_factory=None, **kwargs):
    """
    Create a new database connection.

    The connection parameters can be specified either as a string:

        conn = psycopg2.connect("dbname=test user=postgres password=secret")

    or using a set of keyword arguments:

        conn = psycopg2.connect(database="test", user="postgres", password="secret")

    The basic connection parameters are:

    - *dbname*: the database name (only in dsn string)
    - *database*: the database name (only as keyword argument)
    - *user*: user name used to authenticate
    - *password*: password used to authenticate
    - *host*: database host address (defaults to UNIX socket if not provided)
    - *port*: connection port number (defaults to 5432 if not provided)

    Using the *connection_factory* parameter a different class or connections
    factory can be specified. It should be a callable object taking a dsn
    argument.

    Using the *cursor_factory* parameter, a new default cursor factory will be
    used by cursor().

    Using *async*=True an asynchronous connection will be created. *async_* is
    a valid alias (for Python versions where *async* is a keyword).

    Any other keyword parameter will be passed to the underlying client
    library: the list of supported parameters depends on the library version.

    """
    items = []
    if database is not None:
        items.append(('dbname', database))
    if user is not None:
        items.append(('user', user))
    if password is not None:
        items.append(('password', password))
    if host is not None:
        items.append(('host', host))
    if port is not None:
        items.append(('port', port))

    kwasync = {}
    if 'async' in kwargs:
        kwasync['async'] = kwargs.pop('async')
    if 'async_' in kwargs:
        kwasync['async_'] = kwargs.pop('async_')

    items.extend([(k, v) for (k, v) in kwargs.items() if v is not None])

    if dsn is not None and items:
        raise TypeError(
            "'%s' is an invalid keyword argument when the dsn is specified"
                % items[0][0])

    if dsn is None:
        if not items:
            raise TypeError('missing dsn and no parameters')
        else:
            dsn = " ".join(["%s=%s" % (k, _param_escape(str(v)))
                for (k, v) in items])

    conn = _connect(dsn, connection_factory=connection_factory, **kwasync)
    if cursor_factory is not None:
        conn.cursor_factory = cursor_factory

    return conn



import six

try:
    StandardError = StandardError
except NameError:
    StandardError = Exception


class OperationError(Exception):
    pass

from psycopg2cffi._impl.libpq import libpq, ffi


class Warning(StandardError):
    pass


class Error(StandardError):
    pgerror = None
    pgcode = None
    cursor = None
    _pgres = None

    @property
    def diag(self):
        return Diagnostics(self)

    def __del__(self):
        if self._pgres:
            libpq.PQclear(self._pgres)
            self._pgres = None

    def __reduce__(self):
        t = super(Error, self).__reduce__()
        if not isinstance(t, tuple):
            return t
        # note: in c implementation reduce returns a 2-items tuple;
        # in python a 3-items tuple. Maybe the c exception doesn't have a dict?
        if len(t) != 3:
            return t

        d = t[2].copy()
        d.pop('cursor', None)
        d.pop('_pgres', None)
        return (t[0], t[1], d)

    def __setstate__(self, state):
        self.pgerror = state.get('pgerror')
        self.pgcode = state.get('pgcode')


class InterfaceError(Error):
    pass


class DatabaseError(Error):
    pass


class DataError(DatabaseError):
    pass


class OperationalError(DatabaseError):
    pass


class IntegrityError(DatabaseError):
    pass


class InternalError(DatabaseError):
    pass


class ProgrammingError(DatabaseError):
    pass


class NotSupportedError(DatabaseError):
    pass


class QueryCanceledError(OperationalError):
    pass


class TransactionRollbackError(OperationalError):
    pass


class Diagnostics(object):
    def __init__(self, exc):
        self._exc = exc

    def _get_field(self, field):
        from psycopg2cffi._impl.adapters import bytes_to_ascii
        if self._exc and self._exc._pgres:
            b = libpq.PQresultErrorField(self._exc._pgres, field)
            if b:
                b = ffi.string(b)
                if six.PY3: # py2 tests insist on str here
                    b = bytes_to_ascii(b)
                return b

    @property
    def severity(self):
        return self._get_field(libpq.LIBPQ_DIAG_SEVERITY)

    @property
    def sqlstate(self):
        return self._get_field(libpq.LIBPQ_DIAG_SQLSTATE)

    @property
    def message_primary(self):
        return self._get_field(libpq.LIBPQ_DIAG_MESSAGE_PRIMARY)

    @property
    def message_detail(self):
        return self._get_field(libpq.LIBPQ_DIAG_MESSAGE_DETAIL)

    @property
    def message_hint(self):
        return self._get_field(libpq.LIBPQ_DIAG_MESSAGE_HINT)

    @property
    def statement_position(self):
        return self._get_field(libpq.LIBPQ_DIAG_STATEMENT_POSITION)

    @property
    def internal_position(self):
        return self._get_field(libpq.LIBPQ_DIAG_INTERNAL_POSITION)

    @property
    def internal_query(self):
        return self._get_field(libpq.LIBPQ_DIAG_INTERNAL_QUERY)

    @property
    def context(self):
        return self._get_field(libpq.LIBPQ_DIAG_CONTEXT)

    @property
    def schema_name(self):
        return self._get_field(libpq.LIBPQ_DIAG_SCHEMA_NAME)

    @property
    def table_name(self):
        return self._get_field(libpq.LIBPQ_DIAG_TABLE_NAME)

    @property
    def column_name(self):
        return self._get_field(libpq.LIBPQ_DIAG_COLUMN_NAME)

    @property
    def datatype_name(self):
        return self._get_field(libpq.LIBPQ_DIAG_DATATYPE_NAME)

    @property
    def constraint_name(self):
        return self._get_field(libpq.LIBPQ_DIAG_CONSTRAINT_NAME)

    @property
    def source_file(self):
        return self._get_field(libpq.LIBPQ_DIAG_SOURCE_FILE)

    @property
    def source_line(self):
        return self._get_field(libpq.LIBPQ_DIAG_SOURCE_LINE)

    @property
    def source_function(self):
        return self._get_field(libpq.LIBPQ_DIAG_SOURCE_FUNCTION)




from __future__ import unicode_literals

import re
import six
import base64

from psycopg2cffi._impl import consts
from psycopg2cffi._impl.cursor import Cursor


class Xid(object):
    def __init__(self, format_id, gtrid, bqual):
        if not 0 <= format_id <= 0x7FFFFFFF:
            raise ValueError("format_id must be a non-negative 32-bit integer")

        if len(gtrid) > 64:
            raise ValueError("gtrid must be a string no longer than 64 characters")

        for char in gtrid:
            if not 0x20 <= ord(char) <= 0x7F:
                raise ValueError("gtrid must contain only printable characters")

        if len(bqual) > 64:
            raise ValueError("bqual must be a string no longer than 64 characters")

        for char in bqual:
            if not 0x20 <= ord(char) <= 0x7F:
                raise ValueError("bqual must contain only printable characters")

        self.format_id = format_id
        self.gtrid = gtrid
        self.bqual = bqual

        self.prepared = None
        self.owner = None
        self.database = None

    def as_tid(self):
        if self.format_id is not None:
            gtrid = base64.b64encode(six.b(self.gtrid))
            bqual = base64.b64encode(six.b(self.bqual))
            return "%d_%s_%s" % (int(self.format_id), gtrid.decode(), bqual.decode())
        else:
            return self.gtrid

    def __str__(self):
        return self.as_tid()

    @classmethod
    def from_string(self, s, _re=re.compile("^(\\d+)_([^_]*)_([^_]*)$")):
        m = _re.match(s)
        if m is not None:
            try:
                format_id = int(m.group(1))
                gtrid = base64.b64decode(six.b(m.group(2)))
                bqual = base64.b64decode(six.b(m.group(3)))
                return Xid(format_id, gtrid.decode(), bqual.decode())
            except Exception:
                pass

        # parsing failed: unparsed xid
        xid = Xid(0, '', '')
        xid.gtrid = s
        xid.format_id = None
        xid.bqual = None

        return xid

    def __getitem__(self, idx):
        if idx < 0:
            idx += 3

        if idx == 0:
            return self.format_id
        elif idx == 1:
            return self.gtrid
        elif idx == 2:
            return self.bqual
        raise IndexError("index out of range")

    @classmethod
    def tpc_recover(self, conn):
        # should we rollback?
        rb = conn.status == consts.STATUS_READY and not conn.autocommit

        cur = conn.cursor(cursor_factory=Cursor)
        try:
            cur.execute(
                "SELECT gid, prepared, owner, database "
                "FROM pg_prepared_xacts")

            rv = []
            for gid, prepared, owner, database in cur:
                xid = Xid.from_string(gid)
                xid.prepared = prepared
                xid.owner = owner
                xid.database = database
                rv.append(xid)

            return rv

        finally:
            if rb:
                conn.rollback()




from __future__ import unicode_literals


class Notify(object):
    def __init__(self, pid, channel, payload=''):
        self.pid = pid
        self.channel = channel
        self.payload = payload

    def __eq__(self, other):
        if isinstance(other, tuple):
            return other == self._astuple(False)
        if isinstance(other, Notify):
            return self._astuple(True) == other._astuple(True)
        return False

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        return hash(self._astuple(bool(self.payload)))

    def __getitem__(self, key):
        return (self.pid, self.channel)[key]

    def __len__(self):
        return 2

    def _astuple(self, with_payload):
        if not with_payload:
            return (self.pid, self.channel)
        return (self.pid, self.channel, self.payload)



from psycopg2cffi.extensions import AsIs

def test_asis_with_integer():
    # as documented here: http://initd.org/psycopg/docs/extensions.html#psycopg2.extensions.AsIs
    assert AsIs(42).getquoted() == b'42'



from unittest import TestCase

from psycopg2cffi.extensions import Notify


class TestNotify(TestCase):
    def test_compare(self):

        self.assertTrue(Notify(1, 'foo') == Notify(1, 'foo'))
        self.assertFalse(Notify(1, 'foo') != Notify(1, 'foo'))

        self.assertTrue(Notify(1, 'foo') != Notify(1, 'bar'))

        self.assertTrue(Notify(1, 'foo') != Notify(2, 'foo'))
        self.assertTrue(Notify(1, 'foo') != Notify(2, 'bar'))
        self.assertTrue(Notify(1, 'foo') != Notify(2, 'bar'))

        self.assertTrue(Notify(1, 'foo') != Notify(2, 'bar'))

    def test_compare_payload(self):
        self.assertTrue(Notify(1, 'foo', 'baz') == Notify(1, 'foo', 'baz'))
        self.assertTrue(Notify(1, 'foo') == Notify(1, 'foo', ''))
        self.assertTrue(Notify(1, 'foo', 'foo') != Notify(1, 'foo', 'bar'))

    def test_compare_tuple(self):
        self.assertTrue(Notify(1, 'foo') == (1, 'foo'))
        self.assertTrue(Notify(1, 'foo') != (1, 'bar'))
        self.assertTrue(Notify(1, 'foo') != (2, 'foo'))

    def test_compare_tuple_payload(self):
        self.assertTrue(Notify(1, 'foo', 'baz') == (1, 'foo'))
        self.assertTrue(Notify(1, 'foo', 'baz') != (1, 'bar'))
        self.assertTrue(Notify(1, 'foo', 'baz') != (2, 'foo'))

    def test_indexing(self):
        n = Notify(1, 'foo', 'baz')
        self.assertEqual(n[0], 1)
        self.assertEqual(n[1], 'foo')
        self.assertRaises(IndexError, n.__getitem__, 2)

    def test_len(self):
        n = Notify(1, 'foo', 'baz')
        self.assertEqual(len(n), 2)






from unittest import TestCase

from psycopg2cffi import errorcodes


class TestErrorcodes(TestCase):
    def test_lookup(self):
        self.assertEqual(errorcodes.lookup('54001'), 'STATEMENT_TOO_COMPLEX')
        self.assertRaises(KeyError, errorcodes.lookup, '41234')



# setup.py

import os
import sys

from setuptools import setup


# read the version number from the package
f = open(os.path.join(os.path.dirname(__file__), 'psycopg2cffi/__init__.py'))
try:
    for line in f:
        if line.startswith('__version__'):
            PSYCOPG_VERSION = line.split('=')[1].replace('"', '').replace("'", '').strip()
            break
    else:
        raise ValueError('__version__ not found in psycopg2cffi package')
finally:
    f.close()


README = []
with open('README.rst', 'r') as fh:
    README = fh.readlines()


if '_cffi_backend' in sys.builtin_module_names:   # pypy
    import _cffi_backend
    new_cffi = _cffi_backend.__version__ >= "1"
else:
    new_cffi = True   # assume at least 1.0.0 will be installed


setup_kwargs = dict(
    name='psycopg2cffi',
    author='Konstantin Lopuhin',
    author_email='konstantin.lopuhin@chtd.ru',
    license='LGPL',
    url='http://github.com/chtd/psycopg2cffi',
    version=PSYCOPG_VERSION,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Programming Language :: SQL',
        'Topic :: Database',
        'Topic :: Database :: Front-Ends',
    ],
    platforms=['any'],
    description=README[0].strip(),
    long_description=''.join(README),
    test_suite='psycopg2cffi.tests.suite',
    packages=['psycopg2cffi', 'psycopg2cffi._impl', 'psycopg2cffi.tests'],
    install_requires=['six'],
)

if new_cffi:
    setup_kwargs.update(dict(
        setup_requires=[
            'cffi>=1.0',
            ],
        cffi_modules=['psycopg2cffi/_impl/_build_libpq.py:ffi'],
        install_requires=setup_kwargs['install_requires'] + [
            'cffi>=1.0',
            ],
        ))
else:
    from distutils.command.build_py import build_py as _build_py

    class build_py(_build_py):
        has_been_run = False

        def run(self):
            if not self.dry_run:
                # A hack to import psycopg2cffi._impl._build_libpq
                # without importing psycopg2cffi
                sys.path.append(
                    os.path.join('psycopg2cffi', '_impl'))
                from _build_libpq import _config
                for target_path in [
                        # used for installation
                        os.path.join(self.build_lib, 'psycopg2cffi'),
                        # only for running test from the source tree
                        'psycopg2cffi',
                    ]:
                    self.mkpath(target_path)
                    with open(os.path.join(
                            target_path, '_config.py'), 'w') as f:
                        f.write('# Auto-generated by setup.py\n')
                        f.write('PG_LIB_DIR = %r\n' % _config.libpq_lib_dir)
                        f.write('PG_VERSION = 0x%x\n' % _config.libpq_version)
                        f.write('PG_INCLUDE_DIR = %r\n' %
                            _config.libpq_include_dir)

            _build_py.run(self)
            self.has_been_run = True

    if build_py.has_been_run:
        # building bdist
        from psycopg2cffi._impl.libpq import ffi
        ext_modules = [ffi.verifier.get_extension()]
    else:
        ext_modules = []

    setup_kwargs.update(dict(
        cmdclass={
            'build_py': build_py
        },
        install_requires=setup_kwargs['install_requires'] + [
            'cffi<1.0',
            ],
        setup_requires=[
            'cffi<1.0',
            ],
        ext_package='psycopg2cffi',
        ext_modules=ext_modules,
        ))

setup(**setup_kwargs)



from __future__ import unicode_literals

import os
from functools import wraps
import six

from psycopg2cffi._impl import exceptions
from psycopg2cffi._impl import consts
from psycopg2cffi._impl.libpq import libpq, ffi

INV_WRITE = 0x00020000
INV_READ = 0x00040000


def check_unmarked(func):
    @wraps(func)
    def check_unmarked_(self, *args, **kwargs):
        if self._mark != self._conn._mark:
            raise exceptions.ProgrammingError("lobject isn't valid anymore")
        return func(self, *args, **kwargs)
    return check_unmarked_


def check_closed(func):
    @wraps(func)
    def check_closed_(self, *args, **kwargs):
        if self.closed:
            raise exceptions.InterfaceError("lobject already closed")
        return func(self, *args, **kwargs)
    return check_closed_


class LargeObject(object):
    def __init__(self, conn=None, oid=0, mode='', new_oid=0, new_file=None):
        self._conn = conn
        self._oid = oid
        self._mode = self._parse_mode(mode)
        self._smode = mode
        self._new_oid = new_oid
        self._new_file = new_file
        self._fd = -1
        self._mark = conn._mark

        if conn.autocommit:
            raise exceptions.ProgrammingError(
                "can't use a lobject outside of transactions")
        self._open()

    @property
    def oid(self):
        return self._oid

    @property
    def mode(self):
        return self._smode

    @check_closed
    @check_unmarked
    def read(self, size=-1):
        """Read at most size bytes or to the end of the large object."""
        if size < 0:
            where = self.tell()
            end = self.seek(0, os.SEEK_END)
            self.seek(where, os.SEEK_SET)
            size = end - where

        binary_mode = self._mode & consts.LOBJECT_BINARY
        if size == 0:
            return b'' if binary_mode else ''

        buf = ffi.new('char []', size)
        length = libpq.lo_read(self._conn._pgconn, self._fd, buf, size)
        if length < 0:
            return

        return ffi.buffer(buf, length)[:] if binary_mode else \
               ffi.string(buf).decode(self._conn._py_enc)

    @check_closed
    @check_unmarked
    def write(self, value):
        """Write a string to the large object."""
        if isinstance(value, six.text_type):
            value = value.encode(self._conn._py_enc)
        length = libpq.lo_write(
            self._conn._pgconn, self._fd, value, len(value))
        if length < 0:
            raise self._conn._create_exception()
        return length

    def export(self, file_name):
        """Export large object to given file."""
        self._conn._begin_transaction()
        if isinstance(file_name, six.text_type):
            file_name = file_name.encode(self._conn._py_enc)
        if libpq.lo_export(self._conn._pgconn, self._oid, file_name) < 0:
            raise self._conn._create_exception()

    @check_closed
    @check_unmarked
    def seek(self, offset, whence=0):
        """Set the lobject's current position."""
        return libpq.lo_lseek(self._conn._pgconn, self._fd, offset, whence)

    @check_closed
    @check_unmarked
    def tell(self):
        """Return the lobject's current position."""
        return libpq.lo_tell(self._conn._pgconn, self._fd)

    @check_closed
    @check_unmarked
    def truncate(self, length=0):
        ret = libpq.lo_truncate(self._conn._pgconn, self._fd, length)
        if ret < 0:
            raise self._conn._create_exception()
        return ret

    def close(self):
        """Close and then remove the lobject."""
        if self.closed:
            return True
        if self._conn.autocommit or self._conn._mark != self._mark:
            return True

        ret = libpq.lo_close(self._conn._pgconn, self._fd)
        self._fd = -1
        if ret < 0:
            raise self._conn._create_exception()
        else:
            return True

    @property
    def closed(self):
        return self._fd < 0 or not self._conn or self._conn.closed

    def unlink(self):
        self._conn._begin_transaction()
        self.close()
        libpq.lo_unlink(self._conn._pgconn, self._oid)

    def _open(self):
        conn = self._conn

        conn._begin_transaction()

        if self._oid == 0:
            if self._new_file:
                _new_file = self._new_file
                if isinstance(self._new_file, six.text_type):
                    _new_file = self._new_file.encode(conn._py_enc)
                self._oid = libpq.lo_import(conn._pgconn, _new_file)
            else:
                self._oid = libpq.lo_create(conn._pgconn, self._new_oid)

            self._mode = \
                (self._mode & ~consts.LOBJECT_READ) | consts.LOBJECT_WRITE

        pgmode = 0
        if self._mode & consts.LOBJECT_READ:
            pgmode |= INV_READ
        if self._mode & consts.LOBJECT_WRITE:
            pgmode |= INV_WRITE

        if pgmode:
            self._fd = libpq.lo_open(conn._pgconn, self._oid, pgmode)
            if self._fd < 0:
                raise self._conn._create_exception()

        self._smode = self._unparse_mode(self._mode)

    def _parse_mode(self, smode):
        """Convert a mode string to a mode int"""
        mode = 0
        pos = 0

        binary_default = consts.LOBJECT_TEXT if six.PY3 else \
                consts.LOBJECT_BINARY
        if not smode:
            return consts.LOBJECT_READ | binary_default

        if smode[0:2] == 'rw':
            mode |= consts.LOBJECT_READ | consts.LOBJECT_WRITE
            pos = 2
        else:
            if smode[0] == 'r':
                mode |= consts.LOBJECT_READ
                pos = 1
            elif smode[0] == 'w':
                mode |= consts.LOBJECT_WRITE
                pos = 1
            elif smode[0] == 'n':
                pos = 1
            else:
                mode |= consts.LOBJECT_READ

        if len(smode) > pos:
            if smode[pos] == 't':
                mode |= consts.LOBJECT_TEXT
                pos += 1
            elif smode[pos] == 'b':
                mode |= consts.LOBJECT_BINARY
                pos += 1
            else:
                mode |= binary_default
        else:
            mode |= binary_default

        if len(smode) != pos:
            raise ValueError("bad mode for lobject: '%s'", smode)
        return mode

    def _unparse_mode(self, mode):
        """Convert a mode int to a mode string"""
        smode = ''
        if mode & consts.LOBJECT_READ:
            smode += 'r'
        if mode & consts.LOBJECT_WRITE:
            smode += 'w'
        if not smode:
            smode += 'n'

        if mode & consts.LOBJECT_TEXT:
            smode += 't'
        else:
            smode += 'b'
        return smode
